# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des methodes
class Hermes::Api::Dmo::HousingRequestsController < Hermes::Api::Dmo::ApplicationController
  hermes_tools 'main_user_id'
  hermes_errors 'housing_channel_error'
  
  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ]
  
  ACCEPTABLE_PARAMS = %w[
    ids
    als_id
    is_active
    request_status
    attributed_housing_district
    attributed_housing_district_postal_code
    old_request_status
    request_status_updated_at
    old_entity_status
    updated_entity_status
    entity_status
    dalo
    asked_mobility
    valid_mobility
    decider
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    applicant_ids
    entity_validated
    priority_points_indexes
    nud
    nb_of_requests_made
    reference_tax_revenues
    seeked_districts
    category_type
    single_parent
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_district_postal_codes
    seeked_departments
    seeked_localisation_list
    dependents
    monthly_revenues
    typology_range
    entity_validated
    family_size
    coapplicants
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivation1
    motivation2
    motivation3
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    sne_creation_date
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    min_surface
    main_applicant_id
    main_applicant
    main_employer_siret
    requester_firstname
    requester_lastname
    requester_fullname
    requester_email
    requester_birth_date
    entity_id
    coapplicant_ids
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    housing_offer_ids
    radiation_date
    radiation_motive
    documents_required
    documents_uploaded
    applications_nb
    last_application_date
    is_house_owner
    disability_details
    unborn_children
    owned_housing_details
    actual_housing_details
    postal_details
    has_priority
    available
    is_untensed_area
  ]
  
  REGEX_PARAMS = %w[
  ]
  
  MUTATE_PARAMS = %i[
    workplace_insee_code
    is_active
    attributed_housing_district
    attributed_housing_district_postal_code
    entity_status
    dalo
    asked_mobility
    valid_mobility
    decider
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    applicant_ids
    entity_validated
    priority_points_indexes
    nud
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_localisation_list
    dependents
    entity_validated
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivation1
    motivation2
    motivation3
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    sne_creation_date
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    main_applicant_id
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    radiation_date
    radiation_motive
    documents_required
    documents_uploaded
    is_house_owner
    disability_details
    unborn_children
    owned_housing_details
    actual_housing_details
    postal_details
  ]
  mutate_params_only(*MUTATE_PARAMS)
  
  def include_relationships
    [:housing_applications]
  end
  
  def index
    acceptable_params *ACCEPTABLE_PARAMS
    regex_params *REGEX_PARAMS
    valid_operators *VALID_OPERATORS
    @query_precision ||= {}
    
    # It return only HousingRequest linked to current lessor (with params available = true)
    if current_lessor && params[:available].to_bool
      def include_relationships
        []
      end
      
      request_ids = Hermes::HousingApplication.where(
        :housing_offer_id.in => current_lessor.housing_offer_ids,
        is_active: true
      ).pluck(:housing_request_id).map(&:to_s)
      
      @query_precision[:id] = {'$in': request_ids}
    end
    if is_reservataire && @user.reservation_holder_id
      return paginate(
        index_aggregation(no_acl: true).where(reservation_holder_id: @user.reservation_holder_id),
        each_serializer: @serializer,
        **@options
      )
    end
    if params[:main_employer_siret]
      applicants_ids = Hermes::Applicant.only(:id, :main_employer_siret).where(:main_employer_siret => {'$in' => params[:main_employer_siret]}).pluck(:id)
      @query_precision[:main_applicant_id] = {'$in': applicants_ids}
    end
    super
  end
  
  def is_reservataire
    current_user_roles.include?('Réservataire')
  end
  
  def has_application
    @instance.housing_offer_ids.kind_of?(Array) && @instance.housing_offer_ids.length > 0
  end
  
  def set_instance
    if is_reservataire
      @instance = @model.find_by(id: params[:id], **@query_precision) if params[:id]
    else
      @instance = @model.readable_by(@user).find_by(id: params[:id], **@query_precision) if params[:id]
    end
  end
  
  def user_can_destroy?
    return true if is_reservataire && !reservataire_not_matching_id
    super
  end
  
  def show
    return render_json_error(:not_found, :object_not_found) if !@instance || reservataire_not_matching_id
    if Rails.env != 'test'
    @instance.get_last_xml_from_sne
    @instance.save
    end
    super
  end
  
  def create
    @model.current_user_id = @user.id
    @additional_params = {main_user_id: main_user_id_param} if main_user_id_param
    super
  end
  
  def update
    @model.current_user_id = @user.id
    @additional_params = {main_user_id: main_user_id_param} if main_user_id_param
    return render_json_error(:not_found, :object_not_found) if !@instance || reservataire_not_matching_id
    return render_json_error(:unprocessable_entity, :desactivation_avec_candidature) if mutate_valid_params[:is_active] === false && has_application
    return render_json_error(:unprocessable_entity, :object_is_locked_with_application_because_current_application_existing) if @instance.is_locked_with_application_for(@user)
    
    return check_params_and_update if is_reservataire
    @instance.with_lock(reload: false) do
      super
    end
  end
  
  def update_from_sne
    @model.current_user_id = @user.id
    return render_json_error(:not_found, :object_not_found) if !@instance || reservataire_not_matching_id
    return render_json_error(:unprocessable_entity, :object_is_locked_with_application_because_current_application_existing) if @instance.is_locked_with_application_for(@user)
    return render_json_error(:unprocessable_entity, :update_is_not_possible_because_of_corrupted_sne_info) if @instance.is_locked_with_application_for(@user)
    @instance.update_request_from_sne
    render(
      jsonapi: @instance.reload,
      serializer: @serializer,
      **@options
    )
    
  end
  
  def destroy
    return render_json_error(:not_found, :object_not_found) if !@instance || reservataire_not_matching_id
    return render_json_error(:unprocessable_entity, :suppression_avec_candidature) if has_application
    super
  end
  
  def reservataire_not_matching_id
    is_reservataire && @user.reservation_holder_id != @instance.reservation_holder_id
  end
  
  def clp_details
    return render_json_error(:missing_parameter, :als_ids_param_is_missing) unless params[:als_ids].present?
    
    @options = {detailed_eligibility: true}
    paginate(
      index_aggregation.where(:als_id.in => (params[:als_ids] || [])),
      each_serializer: @serializer,
      **@options
    )
  end
  
  def housing_request_folder
    hrf = @instance.housing_request_folder
    hrf.compute_required_documents
    hrf.save
    render(
      jsonapi: hrf.reload,
      serializer: Hermes::HousingRequestFolderSerializer,
      base_url: request.host,
      **@options
    )
  end
  
  def start_treatment
    hrf = @instance.housing_request_folder
    hrf.start_treatment
    hrf.save
    render body: nil, status: 200
  end
  
  def add_housing_channel
    update_channel do
      @instance.update_pending_channels!(params[:key])
    end
  end
  
  def validate_housing_channel
    update_channel do
      @instance.update_validated_channels!(params[:key])
    end
  end
  
  private
  
  def update_channel
    return render_json_error(:not_found, :instance_not_found) unless @instance
    
    params.require(:key)
    
    if Hermes::HousingChannel.find_by(key: params[:key])
      yield if block_given?
      
      render json: {
        pending_channels: @instance.housing_channel_pending_keys,
        validated_channels: @instance.housing_channel_keys
      }
    else
      render_json_error(:not_found, :instance_not_found, {message: I18n.t('errors.housing_channel.not_found', key: params[:key])})
    end
  
  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, {message: e.message})
  rescue Hermes::Errors::HousingChannelError => e
    render_json_error(:unprocessable_entity, :housing_channel_error, {message: e.message})
  end
end
